searchNodes=[{"doc":"A Goth token server.","ref":"Goth.html","title":"Goth","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Goth.html#child_spec/1","title":"Goth.child_spec/1","type":"function"},{"doc":"Fetches the token, erroring if it is missing. See fetch/2 for more information.","ref":"Goth.html#fetch!/2","title":"Goth.fetch!/2","type":"function"},{"doc":"Fetches the token from the cache. If the token is not in the cache, this function blocks for timeout milliseconds (defaults to 5000 ) while it is attempted to fetch it in the background. To fetch the token bypassing the cache, see Goth.Token.fetch/1 .","ref":"Goth.html#fetch/2","title":"Goth.fetch/2","type":"function"},{"doc":"Starts the server. When the server is started, we attempt to fetch the token and store it in internal cache. If we fail, we'll retry with backoff. Options :name - a unique name to register the server under. It can be any term. :source - the source to retrieve the token from. See documentation for the :source option in Goth.Token.fetch/1 for more information. :refresh_before - Time in seconds before the token is about to expire that it is tried to be automatically refreshed. Defaults to 300 (5 minutes). :http_client - a function that makes the HTTP request. Defaults to using built-in integration with Hackney See documentation for the :http_client option in Goth.Token.fetch/1 for more information. :prefetch - how to prefetch the token when the server starts. The possible options are :async to do it asynchronously or :sync to do it synchronously (that is, the server doesn't start until an attempt to fetch the token was made). Defaults to :async . :max_retries - the maximum number of retries (default: 20 ) :backoff_min - the minimum backoff interval (default: 1_000 ) :backoff_max - the maximum backoff interval (default: 30_000 ) :backoff_type - the backoff strategy, :exp for exponential, :rand for random and :rand_exp for random exponential (default: :rand_exp ) Examples Generate a token using a service account credentials file: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) iex&gt; { :ok , _ } = Goth . start_link ( name : MyApp.Goth , source : { :service_account , credentials , [ ] } ) iex&gt; Goth . fetch! ( MyApp.Goth ) % Goth.Token { ... } Retrieve the token using a refresh token: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) iex&gt; { :ok , _ } = Goth . start_link ( name : MyApp.Goth , source : { :refresh_token , credentials , [ ] } ) iex&gt; Goth . fetch! ( MyApp.Goth ) % Goth.Token { ... } Retrieve the token using the Google metadata server: iex&gt; { :ok , _ } = Goth . start_link ( name : MyApp.Goth , source : { :metadata , [ ] } ) iex&gt; Goth . fetch! ( MyApp.Goth ) % Goth.Token { ... }","ref":"Goth.html#start_link/1","title":"Goth.start_link/1","type":"function"},{"doc":"Functions for retrieving the token from the Google API.","ref":"Goth.Token.html","title":"Goth.Token","type":"module"},{"doc":"Fetch the token from the Google API using the given config . Config may contain the following keys: :source - See &quot;Source&quot; section below. :http_client - a funtion that makes the HTTP request. Can be one of the following: fun - same as {fun, []} {fun, opts} - fun must be a 1-arity funtion that receives a keyword list with fields :method , :url , :headers , and :body along with any passed opts . The funtion must return {:ok, %{status: status, headers: headers, body: body}} or {:error, exception} . See &quot;Custom HTTP Client&quot; section below for more information. fun can also be an atom :hackney to use the built-in Hackney -based client. Defaults to {:hackney, []} . Source Source can be one of: Service account - {:service_account, credentials, options} The credentials is a map and can contain the following keys: &quot;private_key&quot; &quot;client_email&quot; The options is a keywords list and can contain the following keys: :url - the URL of the authentication service, defaults to: &quot;https://www.googleapis.com/oauth2/v4/token&quot; :scopes - the list of token scopes, defaults to [&quot;https://www.googleapis.com/auth/cloud-platform&quot;] (ignored if :claims present) :claims - self-signed JWT extra claims. Should be a map with string keys only. A self-signed JWT will be exchanged for a Google-signed ID token Refresh token - {:refresh_token, credentials, options} The credentials is a map and can contain the following keys: &quot;refresh_token&quot; &quot;client_id&quot; &quot;client_secret&quot; The options is a keywords list and can contain the following keys: :url - the URL of the authentication service, defaults to: &quot;https://www.googleapis.com/oauth2/v4/token&quot; Google metadata server - {:metadata, options} The options is a keywords list and can contain the following keys: :account - the name of the account to generate the token for, defaults to &quot;default&quot; :url - the URL of the metadata server, defaults to &quot;http://metadata.google.internal&quot; :audience - the audience you want an identity token for, default to nil If this parameter is provided, an identity token is returned instead of an access token Custom HTTP Client To use a custom HTTP client, define a function that receives a keyword list with fields :method , :url , :headers , and :body . The funtion must return {:ok, %{status: status, headers: headers, body: body}} or {:error, exception} . Here's an example with Hackney: defmodule MyApp do def request_with_hackney ( options ) do { method , options } = Keyword . pop! ( options , :method ) { url , options } = Keyword . pop! ( options , :url ) { headers , options } = Keyword . pop! ( options , :headers ) { body , options } = Keyword . pop! ( options , :body ) options = [ :with_body ] ++ options case :hackney . request ( method , url , headers , body , options ) do { :ok , status , headers , response_body } -&gt; { :ok , %{ status : status , headers : headers , body : response_body } } { :error , reason } -&gt; { :error , RuntimeError . exception ( inspect ( reason ) ) } end end end And here is how it can be used: iex&gt; Goth.Token . fetch ( %{ source : source , http_client : &amp; MyApp . request_with_hackney / 1 } ) { :ok , % Goth.Token { ... } } iex&gt; Goth.Token . fetch ( %{ source : source , http_client : { &amp; MyApp . request_with_hackney / 1 , connect_timeout : 5000 } } ) { :ok , % Goth.Token { ... } } Examples Generate a token using a service account credentials file: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) iex&gt; Goth.Token . fetch ( %{ source : { :service_account , credentials , [ ] } } ) { :ok , % Goth.Token { ... } } You can generate a credentials file containing service account using gcloud utility like this: $ gcloud iam service-accounts keys create --key-file-type=json --iam-account=... credentials.json Generate a cloud function invocation token using a service account credentials file: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) ...&gt; claims = %{ &quot;target_audience&quot; =&gt; &quot;https://&lt;GCP_REGION&gt;-&lt;PROJECT_ID&gt;.cloudfunctions.net/&lt;CLOUD_FUNCTION_NAME&gt;&quot; } ...&gt; Goth.Token . fetch ( %{ source : { :service_account , credentials , [ claims : claims ] } } ) { :ok , % Goth.Token { ... } } Generate an impersonated token using a service account credentials file: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) ...&gt; claims = %{ &quot;sub&quot; =&gt; &quot;&lt;IMPERSONATED_ACCOUNT_EMAIL&gt;&quot; } ...&gt; Goth.Token . fetch ( %{ source : { :service_account , credentials , [ claims : claims ] } } ) { :ok , % Goth.Token { ... } } Retrieve the token using a refresh token: iex&gt; credentials = &quot;credentials.json&quot; |&gt; File . read! ( ) |&gt; Jason . decode! ( ) iex&gt; Goth.Token . fetch ( %{ source : { :refresh_token , credentials , [ ] } } ) { :ok , % Goth.Token { ... } } You can generate a credentials file containing refresh token using gcloud utility like this: $ gcloud auth application-default login Retrieve the token using the Google metadata server: iex&gt; Goth.Token . fetch ( %{ source : { :metadata , [ ] } } ) { :ok , % Goth.Token { ... } } See Storing and retrieving instance metadata for more information on metadata server. Passing custom Hackney options iex&gt; Goth.Token . fetch ( %{ source : source , http_client : { :hackney , connect_timeout : 5000 } } ) { :ok , % Goth.Token { ... } }","ref":"Goth.Token.html#fetch/1","title":"Goth.Token.fetch/1","type":"function"},{"doc":"","ref":"Goth.Token.html#for_scope/2","title":"Goth.Token.for_scope/2","type":"function"},{"doc":"","ref":"Goth.Token.html#t:t/0","title":"Goth.Token.t/0","type":"type"},{"doc":"Goth Google + Auth = Goth A simple library to generate and retrieve OAuth2 tokens for use with Google Cloud Service accounts.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Note: below are instructions for using Goth v1.3+. For more information on earlier versions of Goth, see v1.2.0 documentation on hexdocs.pm . Add :goth to your list of dependencies in mix.exs . To use the built-in, Hackney-based HTTP client adapter, add :hackney too: def deps do [ { :goth , &quot;~&gt; 1.3-rc&quot; } , { :hackney , &quot;~&gt; 1.17&quot; } ] end Add Goth to your supervision tree: defmodule MyApp.Application do use Application def start ( _type , _args ) do credentials = &quot;GOOGLE_APPLICATION_CREDENTIALS_JSON&quot; |&gt; System . fetch_env! ( ) |&gt; Jason . decode! ( ) source = { :service_account , credentials , [ ] } children = [ { Goth , name : MyApp.Goth , source : source } ] Supervisor . start_link ( children , strategy : :one_for_one ) end end 2.1. ...or use multiple credentials defmodule MyApp.Application do use Application def start ( _type , _args ) do Supervisor . start_link ( load_credentials ( ) , strategy : :one_for_one ) end defp load_credentials do [ { MyApp.Cred1 , &quot;CREDENTIALS_JSON_1&quot; } , ... { MyApp.CredN , &quot;CREDENTIALS_JSON_N&quot; } ] |&gt; Enum . map ( fn { id , env_var } -&gt; credentials = env_var |&gt; System . fetch_env! ( ) |&gt; Jason . decode! ( ) source = { :service_account , credentials , [ ] } Supervisor . child_spec ( { Goth , name : id , source : source } , id : id ) end ) end Fetch the token: iex&gt; Goth . fetch! ( MyApp.Goth ) % Goth.Token { expires : 1453356568 , token : &quot;ya29.cALlJ4ICWRvMkYB-WsAR-CZnExE459PA7QPqKg5nei9y2T9-iqmbcgxq8XrTATNn_BPim&quot; , type : &quot;Bearer&quot; , ... } See Goth.start_link/1 for more information about possible configuration options.","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"See Upgrading from Goth 1.2 guide for more information.","ref":"readme.html#upgrading-from-goth-1-2","title":"README - Upgrading from Goth 1.2","type":"extras"},{"doc":"How to upload on YouTube Data API with elixir ?","ref":"readme.html#community-resources","title":"README - Community resources","type":"extras"},{"doc":"","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Add Goth.start_link/1 , Goth.fetch/2 , and Goth.Token.fetch/1 . Deprecations: Goth.Token : Deprecate for_scope/2 , from_response_json/2 , refresh!/1 Deprecate Goth.Client module Deprecate Goth.Config module Deprecate Goth.TokenStore module Changes from v1.3.0-rc.4 Revert from :refresh_after to :refresh_before Changes from v1.3.0-rc.3 Self-signed JWT claim overrides (#105) Improve retry logic (#127) Improve fetching and prefetching logic (#128) Add :audience option when fetching from metadata server (#121) Simplify the refresh scheduling - replace :refresh_before with :refresh_after (#131) Simplify http client contract (#129) Add Goth.fetch!/2","ref":"changelog.html#v1-3-0-dev-unreleased","title":"CHANGELOG - v1.3.0-dev (unreleased)","type":"extras"},{"doc":"","ref":"upgrade_guide.html","title":"UPGRADE_GUIDE","type":"extras"},{"doc":"Earlier versions of Goth relied on global application environment configuration which is deprecated in favour of a more direct and explicit approach in Goth v1.3+. Previously, Goth depended on on the HTTPoison HTTP client, but now it has an optional dependency on Hackney. Thus, new projects using Goth and wanting to use Hackney will need to explicitly include it in the dependencies. Below is a step-by-step upgrade path from Goth 1.x to 1.3: Change your mix.exs : def deps do [ { :goth , &quot;~&gt; 1.3-rc&quot; } , { :hackney , &quot;~&gt; 1.17&quot; } ] end You might have code similar to this: # config/config.exs config :goth , json : { :system , &quot;GCP_CREDENTIALS&quot; } # lib/myapp.ex defmodule MyApp do def gcloud_authorization ( ) do { :ok , token } = Goth.Token . for_scope ( &quot;https://www.googleapis.com/auth/cloud-platform.read-only&quot; ) &quot; \#{ token . type } \#{ token . token } &quot; end end Replace it with: defmodule MyApp.Application do @moduledoc false use Application def start ( _type , _args ) do credentials = &quot;GCP_CREDENTIALS&quot; |&gt; System . fetch_env! ( ) |&gt; Jason . decode! ( ) scopes = [ &quot;https://www.googleapis.com/auth/cloud-platform.read-only&quot; ] source = { :service_account , credentials , scopes : scopes } children = [ { Goth , name : MyApp.Goth , source : source } ] Supervisor . start_link ( children , strategy : :one_for_one ) end end # lib/myapp.ex defmodule MyApp do def gcloud_authorization ( ) do { :ok , token } = Goth . fetch ( MyApp.Goth ) &quot; \#{ token . type } \#{ token . token } &quot; end end For more information on earlier versions of Goth, see v1.2.0 documentation on hexdocs.pm .","ref":"upgrade_guide.html#upgrading-from-goth-1-2","title":"UPGRADE_GUIDE - Upgrading from Goth 1.2","type":"extras"}]